﻿using Kivii.EmailPools.Entities;
using Kivii.Linq;
using System;
using System.Collections.Generic;
using System.Data;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Mail;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.EmailPools
{
    public static class Extensions
    {
        private static Dictionary<string, SmtpConfig> _smtpConfigs = new Dictionary<string, SmtpConfig>();

        /// <summary>
        /// 获取Smtp配置
        /// </summary>
        /// <param name="FromAddress"></param>
        /// <returns></returns>
        public static SmtpConfig GetSmtpConfig(this string FromAddress)
        {
            //默认key
            var key = "default";
            if (!FromAddress.IsNullOrEmpty())
            {
                try
                {
                    //验证传入的邮箱地址格式是否正确，正确就赋值key，不正确key取默认值
                    var addr = new System.Net.Mail.MailAddress(FromAddress);
                    key = FromAddress;
                }
                catch { }
            }
            if (_smtpConfigs.ContainsKey(key))
            {
                return _smtpConfigs[key];
            }
            var conn = KiviiContext.GetOpenedDbConnection<SmtpConfig>();

            var smtpConfigs = conn.Select<SmtpConfig>();
            if (!smtpConfigs.IsNullOrEmpty())
            {
                var defaultConfig = smtpConfigs.FirstOrDefault(o => o.IsDefault);
                if (defaultConfig == null) defaultConfig = smtpConfigs.FirstOrDefault();
                _smtpConfigs["default"] = defaultConfig;
                foreach (var item in smtpConfigs)
                {
                    _smtpConfigs[item.Address] = item;
                }
            }
            if (_smtpConfigs.ContainsKey(key))
            {
                return _smtpConfigs[key];
            }
            return null;
        }

        /// <summary>
        /// 通过emialPool找到对应的附件
        /// </summary>
        /// <param name="pool"></param>
        /// <returns></returns>
        public static List<Attachment> GetAttachments(this EmailPool pool)
        {
            var conn=KiviiContext.GetOpenedDbConnection<EmailPool>();
            var query = conn.From<EntityDbFile<EmailPool>>();
            query.Where(o => o.ParentKvid == Guid.Empty&&o.DbFolderPath==Configs.EmailPoolDbFolderPath && o.OwnerKvid == pool.Kvid);
            var dbFiles = conn.Select(query);
            if (dbFiles.IsNullOrEmpty()) return null;

            var rtns = new List<Attachment>();

            foreach (var dbFile in dbFiles)
            {
                //获取到文件的实际存储路径
                var physicalStorageFilePath = dbFile.GetPhysicalPath();
                if (!File.Exists(physicalStorageFilePath)) continue;//判断此文件是否存在，不存在就跳过
                var attach = new Attachment(physicalStorageFilePath);
                rtns.Add(attach);
            }
            return rtns;
        }


        /// <summary>
        /// 发送email
        /// </summary>
        /// <param name="pool"></param>
        /// <returns></returns>
        /// <exception cref="Exception"></exception>
        public static EmailPool Sending(this EmailPool pool)
        {
            var config = pool.FromAddress.GetSmtpConfig();
            if (config == null) throw new Exception("未找到指定SMTP配置，请先配置SMTP服务信息");
            MailAddress from = null;
            MailAddress to = null;
            try
            {
                from = new MailAddress(config.Address, config.Name);
            }
            catch
            {
                throw new Exception($"发送地址{config.Address},格式错误");
            }
            try
            {
                to = new MailAddress(pool.ToAddress, pool.ToName);
            }
            catch
            {
                throw new Exception($"收件地址{pool.ToAddress},格式错误");
            }
            SmtpClient smtp = new SmtpClient(config.SmtpUrl);
            smtp.UseDefaultCredentials = false;
            smtp.Credentials = new NetworkCredential(config.UserId, config.Password);
            smtp.DeliveryMethod = SmtpDeliveryMethod.Network;
            smtp.EnableSsl = true;

            MailMessage msg = new MailMessage();
            msg.From = from;
            msg.To.Add(to);
            msg.BodyEncoding = Encoding.UTF8;
            msg.Body = pool.Body;
            msg.Subject = pool.Subject;
            msg.IsBodyHtml = pool.IsBodyHtml;
            var attachs = pool.GetAttachments();
            if (!attachs.IsNullOrEmpty())
            {
                foreach (var attach in attachs)
                {
                    msg.Attachments.Add(attach);
                }
            }
            try
            {
                smtp.Send(msg);
                pool.SendStatus = SendStatus.Complete;
                pool.AddOnlyProperties(o => o.SendStatus);
            }
            catch (SmtpException ex)
            {
                pool.SendStatus = SendStatus.Fail;
                pool.AddOnlyProperties(o => o.SendStatus);

                pool.Remark = ex.Message;
                pool.AddOnlyProperties(o => o.Remark);
            }
            finally
            {
                pool.IsSended = true;
                pool.AddOnlyProperties(o => o.IsSended);
            }
            return pool;
        }
    }
}
