﻿using Kivii.Configuration;
using Kivii.EmailPools.Entities;
using Kivii.Linq;
using Kivii.Web;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Net.Mail;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.EmailPools.Transforms
{
    [Api(Description = "邮箱池创建")]
    [RequiresAnyRole(MemberRoles.Everyone)]
    public class EmailPoolCreate : RestfulExecution<EmailPool>
    {
        public EmailPool Item { get; set; }
        public List<EmailPool> Items { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            (Item == null && Items.IsNullOrEmpty()).ThrowIfTrue("Need Item Or Items!");

            if (Items.IsNullOrEmpty()) Items = new List<EmailPool>();
            if (Item != null) Items.Add(Item);

            var conn = KiviiContext.GetOpenedDbConnection<EmailPool>();
            var rtns = new RestfulCreateResponse<EmailPool>();
            rtns.Results = new List<EmailPool>();

            foreach (var item in Items)
            {
                try
                {
                    try
                    {
                        var from = new MailAddress(item.FromAddress, item.FromAddress);
                    }
                    catch
                    {
                        throw new Exception($"发送地址{item.FromAddress},格式错误");
                    }
                    try
                    {
                        var to = new MailAddress(item.ToAddress, item.ToName);
                    }
                    catch
                    {
                        throw new Exception($"收件地址{item.ToAddress},格式错误");
                    }
                    conn.Insert(item);
                    rtns.Results.Add(item);
                }
                catch
                {
                    continue;
                }
            }
            return rtns;
        }
    }

    [Api(Description = "邮箱池更新")]
    [RequiresAnyRole(MemberRoles.Everyone)]
    public class EmailPoolUpdate : RestfulUpdate<EmailPool>
    {
        public override bool OnPreRestfulUpdate(IRequest req, IResponse res, IDbConnection dbConnection, IRestfulUpdateResponse<EmailPool> rtns)
        {
            var kvids = Items.ConvertAll(p => p.Kvid);
            if(dbConnection.Exists<EmailPool>(o => o.IsSended && Sql.In(o.Kvid, kvids))) throw new Exception("存在已发送的数据,无法更新数据!");

            foreach (var item in Items)
            {
                if (item.OnlyPropertiesIsExist(o=>o.FromAddress))
                {
                    try
                    {
                        var from = new MailAddress(item.FromAddress, item.FromAddress);
                    }
                    catch
                    {
                        throw new Exception($"发送地址{item.FromAddress},格式错误");
                    }
                }

                if (item.OnlyPropertiesIsExist(o => o.ToAddress))
                {
                    try
                    {
                        var to = new MailAddress(item.ToAddress, item.ToName);
                    }
                    catch
                    {
                        throw new Exception($"收件地址{item.ToAddress},格式错误");
                    }
                }
            }
            return base.OnPreRestfulUpdate(req, res, dbConnection, rtns);
        }
    }

    [Api(Description = "邮箱池查询")]
    [RequiresAnyRole(MemberRoles.Everyone)]
    public class EmailPoolQuery : RestfulQuery<EmailPool>
    {
        
    }

    [Api(Description = "未发送邮箱池")]
    [RequiresAnyRole(MemberRoles.Everyone)]
    public class EmailPoolPreSending : RestfulExecution<EmailPool>
    {
        public override object OnExecution(IRequest req, IResponse res)
        {
            var conn = KiviiContext.GetOpenedDbConnection<EmailPool>();
            var pools = conn.Select<EmailPool>(o => o.IsSended == false);

            var rtns = new RestfulQueryResponse<EmailPool>();
            rtns.Results = new List<EmailPool>();
            rtns.Results.AddRange(pools);
            rtns.Total = rtns.Results.Count;

            return rtns;
        }
    }

    [Api(Description = "发送邮箱")]
    [RequiresAnyRole(MemberRoles.Everyone)]
    public class EmailPoolSending : RestfulExecution<EmailPool>
    {
        public List<Guid> Kvids { get; set; }


        public override object OnExecution(IRequest req, IResponse res)
        {
            var conn = KiviiContext.GetOpenedDbConnection<EmailPool>();
            var pools = conn.SelectByIds<EmailPool>(Kvids);

            var rtns = new RestfulExecutionResponse<EmailPool>();
            rtns.Results = new List<EmailPool>();

            if (pools.IsNullOrEmpty()) return rtns;

            foreach (var item in pools)
            {
                var result = item.Sending();
                conn.UpdateOnly(result);
                rtns.Results.Add(result);
            }

            return rtns;
        }
    }

    [Api(Description = "标记发送完成")]
    [RequiresAnyRole(MemberRoles.Everyone)]
    public class EmailPoolComplete : RestfulExecution<EmailPool>
    {
        public List<Guid> Kvids { get; set; }
        public string OperatorName { get; set; }
        public Guid OperatorKvid { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            Kvids.ThrowIfNullOrEmpty("Need Kvids");

            var conn = KiviiContext.GetOpenedDbConnection<EmailPool>();
            var pools = conn.SelectByIds<EmailPool>(Kvids);
            pools.ThrowIfNullOrEmpty($"EmailPool Not Found:Kvid={Kvids}");

            var rtns = new RestfulExecutionResponse<EmailPool>();
            rtns.Results = new List<EmailPool>();

            foreach (var item in pools)
            {
                item.IsSended = true;
                item.AddOnlyProperties(o => o.IsSended);
                item.SendStatus = SendStatus.Complete;
                item.AddOnlyProperties(o => o.SendStatus);
                item.OperatorName = OperatorName;
                item.AddOnlyProperties(o => o.OperatorName);
                item.OperatorKvid = OperatorKvid;
                item.AddOnlyProperties(o => o.OperatorKvid);
                //item.Category = "发送成功";
                //item.AddOnlyProperties(o => o.Category);
                //item.Remark = "";
                //item.AddOnlyProperties(o => o.Remark);
                conn.UpdateOnly(item);
                rtns.Results.Add(item);

                var log = new EntityLog<EmailPool>();
                log.OwnerKvid = item.OwnerKvid;
                log.BizId = item.OwnerName;
                log.BizKvid = item.OwnerKvid;
                log.BizType = typeof(EmailPool).FullName;
                log.Type = "电子邮箱";
                log.Title = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
                log.Summary = "电子邮箱发送完成";
                log.Remark = $"[发送成功] 已成功将{item.OwnerName} 发送至客户邮箱({item.ToAddress}). 时间:{DateTime.Now:yyyy-MM-dd HH:mm:ss}!";
                conn.Insert(log);
            }

            return rtns;
        }
    }

    [Api(Description = "标记发送失败")]
    [RequiresAnyRole(MemberRoles.Everyone)]
    public class EmailPoolFail : RestfulExecution<EmailPool>
    {
        public List<EmailPool> Items { get; set; }

        public string OperatorName { get; set; }
        public Guid OperatorKvid { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            Items.ThrowIfNullOrEmpty("Need Kvids");

            var conn = KiviiContext.GetOpenedDbConnection<EmailPool>();
            var pools = conn.SelectByIds<EmailPool>(Items.ConvertAll(p => p.Kvid));
            pools.ThrowIfNullOrEmpty($"EmailPools Not Found!");

            var rtns = new RestfulExecutionResponse<EmailPool>();
            rtns.Results = new List<EmailPool>();

            foreach (var item in pools)
            {
                var current = Items.FirstOrDefault(o => o.Kvid == item.Kvid);
                if (current == null) continue;
                item.IsSended = true;
                item.AddOnlyProperties(o => o.IsSended);
                item.SendStatus = SendStatus.Fail;
                item.AddOnlyProperties(o => o.SendStatus);
                item.OperatorName = OperatorName;
                item.AddOnlyProperties(o => o.OperatorName);
                item.OperatorKvid = OperatorKvid;
                item.AddOnlyProperties(o => o.OperatorKvid);
                //item.Category = $"发送失败";
                //item.AddOnlyProperties(o => o.Category);
                item.Remark = current.Remark;
                item.AddOnlyProperties(o => o.Remark);
                conn.UpdateOnly(item);
                rtns.Results.Add(item);

                var log = new EntityLog<EmailPool>();
                log.OwnerKvid = item.OwnerKvid;
                log.BizId = item.OwnerName;
                log.BizKvid = item.OwnerKvid;
                log.BizType = typeof(EmailPool).FullName;
                log.Type = "电子邮箱";
                log.Title = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
                log.Summary = "电子邮箱发送失败";
                log.Remark = $"[发送失败] 发送{item.ToAddress}失败原因:{item.Remark}. 时间:{DateTime.Now:yyyy-MM-dd HH:mm:ss}!";
                conn.Insert(log);
            }

            return rtns;
        }
    }

    [Api(Description = "取消发送")]
    [RequiresAnyRole(MemberRoles.Everyone)]
    public class EmailPoolCancel : RestfulExecution<EmailPool>
    {
        public List<Guid> Kvids { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            Kvids.ThrowIfNullOrEmpty("Need Kvids");

            var conn = KiviiContext.GetOpenedDbConnection<EmailPool>();
            var pools = conn.SelectByIds<EmailPool>(Kvids);
            pools.ThrowIfNullOrEmpty($"EmailPools Not Found!");

            var rtns = new RestfulExecutionResponse<EmailPool>();
            rtns.Results = new List<EmailPool>();

            foreach (var item in pools)
            {
                if (item.IsSended) continue;
                item.IsSended = true;
                item.AddOnlyProperties(o => o.IsSended);
                item.SendStatus = SendStatus.Cancel;
                item.AddOnlyProperties(o => o.SendStatus);
                item.OperatorName = KiviiContext.CurrentMember.FullName;
                item.AddOnlyProperties(o => o.OperatorName);
                item.OperatorKvid = KiviiContext.CurrentMember.Kvid;
                item.AddOnlyProperties(o => o.OperatorKvid);
                //item.Category = $"取消发送";
                //item.AddOnlyProperties(o => o.Category);
                conn.UpdateOnly(item);
                rtns.Results.Add(item);

                var log = new EntityLog<EmailPool>();
                log.OwnerKvid = item.OwnerKvid;
                log.BizId = item.OwnerName;
                log.BizKvid = item.OwnerKvid;
                log.BizType = typeof(EmailPool).FullName;
                log.Type = "电子邮箱";
                log.Title = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
                log.Summary = "取消电子邮箱发送";
                log.Remark = $"[取消发送] 操作人:{KiviiContext.CurrentMember.FullName}. 时间:{DateTime.Now:yyyy-MM-dd HH:mm:ss}!";
                conn.Insert(log);
            }

            return rtns;
        }
    }
}
